% =============================================================================================
% Finds all the possible action effects (direct or indirect).
% =============================================================================================

:- dynamic action_effects/2.
:- dynamic effects_history/2.


find_action_effects(Rams, Actions):-
	find_direct_effects(Actions, Level0),
	find_all_levels_indirect_effects(Rams, Level0, AllLevels),
	save_action_effects(AllLevels).


find_direct_effects(Actions, Level0):-
	findall(effect(Effect,A,T,GroundActPrec,[]),
		(	mymember(action(happens(A,T),Effect,ActPrec),Actions),
			clause(happens(A,T)),
			ground(ActPrec,GroundActPrec)
		),
      	Level),
	removeduplicates(Level, Level0).


find_all_levels_indirect_effects(Rams, Level0, AllLevels):-
	find_all_levels_indirect_effects(Rams, Level0, Level0, AllLevels).

find_all_levels_indirect_effects(_Rams, _Acc, [], []):- !.
find_all_levels_indirect_effects(Rams, Acc, InLevel, [InLevel|RestLevels]):-
	find_next_level_indirect_effects(Rams, InLevel, NextLevel),
	myappend(NextLevel, Acc, AllEffects),
	remove_extra(AllEffects, NewAcc),
	intersection(NextLevel, NewAcc, NewNextLevel),
	find_all_levels_indirect_effects(Rams, NewAcc, NewNextLevel, RestLevels).


find_next_level_indirect_effects(Rams, PreviousLevel, ThisLevel):-
	findall(effect(RamH,A,T,ActPrec,NewAssumptions),
		(	mymember(effect(Effect,A,T,ActPrec,Assumptions),PreviousLevel),
			mymember(ramification(RamH,RamBody),Rams),
			RamH \= neg(holds(true,_)),
			myselect(Effect,RamBody,RestRamBody),
			ground(RestRamBody,GroundRamBody),
			myappend(GroundRamBody,Assumptions,NewAssumptions1),
			removeduplicates(NewAssumptions1,NewAssumptions)
		),
      	ThisLevel1),
	removeduplicates(ThisLevel1, ThisLevel).


remove_extra(AllEffects, NewEffects):-
	myselect(effect(Effect,_,_,ActPrec1,Assumptions1), AllEffects, RestEffects),
	mymember(effect(Effect,_,_,ActPrec2,Assumptions2), RestEffects),
	subset(ActPrec2,ActPrec1), subset(Assumptions2,Assumptions1), !,
	remove_extra(RestEffects, NewEffects).
remove_extra(RestEffects, RestEffects).


save_action_effects(AllEffects):-
	assert(action_effects(dummy,dummy)),
	retract_all(action_effects(_,_)),
	save_action_effects(AllEffects, 0).

save_action_effects([], _).
save_action_effects([Level|RestLevels], N):-
	assert(action_effects(N, Level)),
	show(['Level: ', N, '\n']), showList(Level),
	N1 is N+1,
	save_action_effects(RestLevels, N1).



% Recall and use action effects.
get_action_effect_info(Effect, T, ActPrec, Assumptions):-
	clause(action_effects(_, Level)),
	mymember(effect(Effect, _A, T, ActPrec, Assumptions), Level),
	loiprint(effect(Effect, _A, T, ActPrec, Assumptions)).

%loiprint(effect(holds(in_position_for_mounting(john, elly),_), _A, T, ActPrec, Assumptions)):- !,
%		say([effect(holds(in_position_for_mounting(john,elly)), _A, T, ActPrec, Assumptions), '\n']).
%loiprint(effect(neg(holds(in_position_for_mounting(john, elly),_)), _A, T, ActPrec, Assumptions)):- !,
%		say([effect(neg(holds(in_position_for_mounting(john,elly))), _A, T, ActPrec, Assumptions), '\n']).
loiprint(_).


% =============================================================================================
% End of file.
% =============================================================================================
