% Needs instantiated ramifications.


% =============================================================================================
% Finds which Literals are forced to a value given a Seed set and the Ramifications.
% =============================================================================================

find_forced_literals(Rams, SeedLiterals, ForcedLiterals):-
	find_forced_literals(Rams, [holds(true,_)|SeedLiterals], ForcedLiterals, _).

find_forced_literals(Rams, SeedLiterals, ForcedLiterals, UnsatisfiedRams):-
	copy_term(Rams, CopyOfRams), % Protect time variables from unification.
	find_forced_literals_1(CopyOfRams, SeedLiterals, ForcedLiterals, UnsatisfiedRams).


find_forced_literals_1(Rams, SeedLiterals, ForcedLiterals, UnsatisfiedRams):-
	find_forced_literals_one_pass(Rams, SeedLiterals, OnePass, UnsatisfiedRams1), !,
	find_forced_literals_2(UnsatisfiedRams1, SeedLiterals, ForcedLiterals, OnePass, UnsatisfiedRams).
find_forced_literals_1(_, _, _, _):-
	show('>> Clash in forced literals.\n'),
	fail.
	
find_forced_literals_2(Rams, SeedLiterals, ForcedLiterals, OnePass, UnsatisfiedRams):-
   	subtract(OnePass, SeedLiterals, [_|_]), !,
	find_forced_literals_1(Rams, OnePass, ForcedLiterals, UnsatisfiedRams).
find_forced_literals_2(Rams, SeedLiterals, SeedLiterals, _OnePass, Rams).


find_forced_literals_one_pass([], SeedLiterals, SeedLiterals, []).
find_forced_literals_one_pass([ramification(H,_B)|Rams], SeedLiterals, ForcedLiterals, UnsatisfiedRams):-
	mymember(H, SeedLiterals), !,
	find_forced_literals_one_pass(Rams, SeedLiterals, ForcedLiterals, UnsatisfiedRams).
find_forced_literals_one_pass([ramification(_H,B)|Rams], SeedLiterals, ForcedLiterals, UnsatisfiedRams):-
	mymember(M, B), contrary(M, NM), mymember(NM, SeedLiterals), !,
	find_forced_literals_one_pass(Rams, SeedLiterals, ForcedLiterals, UnsatisfiedRams).
find_forced_literals_one_pass([ramification(H,B)|Rams], SeedLiterals, ForcedLiterals, UnsatisfiedRams):-
   	subtract(B, SeedLiterals, []), !,
	is_consistent(H, SeedLiterals),
	find_forced_literals_one_pass(Rams, [H|SeedLiterals], ForcedLiterals, UnsatisfiedRams).
find_forced_literals_one_pass([ramification(H,B)|Rams], SeedLiterals, ForcedLiterals, UnsatisfiedRams):-
	mymember(M, B), subtract(B, [M], RestBody), contrary(H, NH),
   	subtract([NH|RestBody], SeedLiterals, []), !,
	contrary(M, NM), is_consistent(NM, SeedLiterals),
	find_forced_literals_one_pass(Rams, [NM|SeedLiterals], ForcedLiterals, UnsatisfiedRams).
find_forced_literals_one_pass([R|Rams], SeedLiterals, ForcedLiterals, [R|UnsatisfiedRams]):-
	find_forced_literals_one_pass(Rams, SeedLiterals, ForcedLiterals, UnsatisfiedRams).


% =============================================================================================
% End of file.
% =============================================================================================
