
proc translate_domain {} {
   upvar domain_name domain_name_local
   set targetDomain ""
   set domainDescription  [.domain_description get 0.0 end]
   set linesArray [split $domainDescription "\r\n"]
   set actionsDescription [.actions_description get 0.0 end]
   set actionsArray [split $actionsDescription "\r\n"]

   destroy .translation
   toplevel .translation
   wm title .translation "Translation of Language E"

   foreach line $linesArray {
	set targetLine [parse_domain $line]
        if {[string compare $targetLine ""] != 0} {lappend targetDomain $targetLine}
   }

   foreach line $actionsArray {
        set actionLine [parse_action $line]
        if {[string compare $actionLine ""] != 0} {lappend targetDomain $actionLine}
   }

   set targetDomain [lsort $targetDomain]
   set targetDomain [join $targetDomain "\n"]

   frame .translation.up
   frame .translation.down

   button .translation.more -text "More" -command {ec_queue_write toplevel_in [ec_tcl2exdr more ()];resume_query} -state disabled
   label .translation.status -bg white -relief sunken -width 20 -textvariable ec_state
   label .translation.statuslabel -relief sunken -width 10 -text "Status:"


   pack .translation.up .translation.down -side top -fill both -expand true
   pack .translation.statuslabel .translation.status -side left
   pack .translation.more -fill x


   scrollbar .translation.up.ds -orient vertical -command ".translation.up.domain yview"
   label .translation.up.labelDomain -text "Translation of Language E Domain $domain_name_local"
   text .translation.up.domain -height 10 -width 50 -yscrollcommand ".translation.up.ds set"
   pack .translation.up.labelDomain -side top -fill x
   pack .translation.up.domain -side left -fill both -expand true
   pack .translation.up.ds -side left -fill y

   .translation.up.domain insert end $targetDomain

   # Save the domain translation to a temporary file.
   set fileID [open domain.pl w]
   puts $fileID {:-retract_all(initiation(_, _)).}
   puts $fileID {:-retract_all(termination(_, _)).}
   puts $fileID {:-retract_all(happens(_, _)).}
   puts $fileID {:-retract_all(tprop(_)).}
   puts $fileID {:-retract_all(ram(_)).}
   puts $fileID {:-retract_all(abducible(_)).}
   puts $fileID {:-retract_all(constantfluent(_)).}
   puts $fileID {:-retract_all(pprop(_, _, _)).}
   puts $fileID $targetDomain
   close $fileID

   queryEclipse "compile('domain')"
   queryEclipse "domain_pre_processing"

   .translation.up.domain configure -state disabled
}


proc assimilate_query {} {
   upvar domain_name domain_name_local
   upvar query_translate translateAs
   
   scrollbar .translation.down.qs -orient vertical -command ".translation.down.query yview"
   label .translation.down.labelQuery -text "Translation of queries for Language E Domain $domain_name_local"
   text .translation.down.query -height 10 -width 59 -yscrollcommand ".translation.down.qs set"
   pack .translation.down.labelQuery -side top -fill x
   pack .translation.down.query -side left -fill both -expand true
   pack .translation.down.qs -side left -fill y

   button .translation.down.query.assimilate -text "Assimilate Observations" -command "queryEclipse {credulous(\[\], Explanation).}" -cursor arrow
   .translation.down.query window create end -window .translation.down.query.assimilate
   .translation.down.query insert end "\n\n"

   .translation.down.query configure -state disabled

}


proc translate_query {} {
   upvar domain_name domain_name_local
   upvar query_translate translateAs
   set targetQuery ""
   set queryDescription [.query_description get 0.0 end]
   set queryLines [split $queryDescription "\r\n"]

   scrollbar .translation.down.qs -orient vertical -command ".translation.down.query yview"
   label .translation.down.labelQuery -text "Translation of queries for Language E Domain $domain_name_local"
   text .translation.down.query -height 10 -width 59 -yscrollcommand ".translation.down.qs set"
   pack .translation.down.labelQuery -side top -fill x
   pack .translation.down.query -side left -fill both -expand true
   pack .translation.down.qs -side left -fill y

   button .translation.down.query.assimilate -text "Assimilate Observations" -command "queryEclipse {credulous(\[\], Explanation).}" -cursor arrow
   .translation.down.query window create end -window .translation.down.query.assimilate
   .translation.down.query insert end "\n\n"



   set i 0
   foreach line $queryLines {
	set targetQueryLine [parse_query $line]
        if {[string compare $targetQueryLine ""] != 0} {
	     	if {[string compare $translateAs separate] == 0} {
		    incr i

		    label ".translation.down.query.label_${i}" -text "E-RES queries for $line " -foreground red

		    .translation.down.query window create end -window .translation.down.query.label_${i}
		    .translation.down.query insert end "\n\n"

		    global targetQuery_${i}_1 targetQuery_${i}_2 targetQuery_${i}_3

		    set targetQuery_${i}_1 "sceptical(\[$targetQueryLine\])."
		    set targetQuery_${i}_2 "credulous(\[$targetQueryLine\])."
		    set targetQuery_${i}_3 "credulous(\[$targetQueryLine\], Explanation)."

		    entry .translation.down.query.entry_${i}_1 -textvariable targetQuery_${i}_1 -width 50
		    entry .translation.down.query.entry_${i}_2 -textvariable targetQuery_${i}_2 -width 50
		    entry .translation.down.query.entry_${i}_3 -textvariable targetQuery_${i}_3 -width 50

		    button .translation.down.query.submit_${i}_1 -text "Submit" -command "queryEclipse \$targetQuery_${i}_1" -cursor arrow
		    button .translation.down.query.submit_${i}_2 -text "Submit" -command "queryEclipse \$targetQuery_${i}_2" -cursor arrow
		    button .translation.down.query.submit_${i}_3 -text "Submit" -command "queryEclipse \$targetQuery_${i}_3" -cursor arrow

		    .translation.down.query window create end -window .translation.down.query.entry_${i}_1
		    .translation.down.query window create end -window .translation.down.query.submit_${i}_1
		    .translation.down.query insert end "\n"

		    .translation.down.query window create end -window .translation.down.query.entry_${i}_2
		    .translation.down.query window create end -window .translation.down.query.submit_${i}_2
		    .translation.down.query insert end "\n"

		    .translation.down.query window create end -window .translation.down.query.entry_${i}_3
		    .translation.down.query window create end -window .translation.down.query.submit_${i}_3
		    .translation.down.query insert end "\n\n"
		} else {
		    lappend targetQuery $targetQueryLine
		}
	}
   }


   if {[string compare $translateAs compound] == 0} {
      label .translation.down.query.label_compound -text "Associated E-RES queries:" -foreground red

      .translation.down.query window create end -window .translation.down.query.label_compound
      .translation.down.query insert end "\n\n"

      set tempStr $targetQuery
      set tempStr [join $tempStr ,]

      global targetQuery_${i}_1 targetQuery_${i}_2 targetQuery_${i}_3

      set targetQuery_${i}_1 "sceptical(\[$tempStr\])."
      set targetQuery_${i}_2 "credulous(\[$tempStr\])."
      set targetQuery_${i}_3 "credulous(\[$tempStr\], Explanation)."

      entry .translation.down.query.entry_${i}_1 -textvariable targetQuery_${i}_1 -width 50
      entry .translation.down.query.entry_${i}_2 -textvariable targetQuery_${i}_2 -width 50
      entry .translation.down.query.entry_${i}_3 -textvariable targetQuery_${i}_3 -width 50

      button .translation.down.query.submit_${i}_1 -text "Submit" -command "queryEclipse \$targetQuery_${i}_1" -cursor arrow
      button .translation.down.query.submit_${i}_2 -text "Submit" -command "queryEclipse \$targetQuery_${i}_2" -cursor arrow
      button .translation.down.query.submit_${i}_3 -text "Submit" -command "queryEclipse \$targetQuery_${i}_3" -cursor arrow

      .translation.down.query window create end -window .translation.down.query.entry_${i}_1
      .translation.down.query window create end -window .translation.down.query.submit_${i}_1
      .translation.down.query insert end "\n"

      .translation.down.query window create end -window .translation.down.query.entry_${i}_2
      .translation.down.query window create end -window .translation.down.query.submit_${i}_2
      .translation.down.query insert end "\n"

      .translation.down.query window create end -window .translation.down.query.entry_${i}_3
      .translation.down.query window create end -window .translation.down.query.submit_${i}_3
      .translation.down.query insert end "\n\n"


   }

   

   .translation.down.query configure -state disabled
}



proc parse_query { queryLine } {
   set literalPattern {-?[\_\=\<\>\\a-zA-Z0-9]+} ;# {-?[a-zA-Z][a-zA-Z0-9]*}
   set numberPattern {[0-9]+}
   set compoundLiteralPattern "${literalPattern}(\\(\[ \t\]*${literalPattern}\[ \t\]*(,\[ \t\]*${literalPattern})*\[ \t\]*\\))?"
   set compoundLiteralPattern "${literalPattern}(\\(\[ \t\]*${literalPattern}\[ \t\]*(,\[ \t\]*${compoundLiteralPattern})*\[ \t\]*\\))?"

   set tPattern "^\[ \t\]*(${compoundLiteralPattern})\[ \t\]+(holds-at)\[ \t\]+(${numberPattern})\[ \t\]*\$"

   set targetQueryLine ""

   switch -regexp -- $queryLine \
	$tPattern {set targetQueryLine [tQueryTranslate $queryLine]} \
	"^\[ \t\]*\$" {} \
	default {set targetQueryLine "Syntax error: $queryLine"}

   return $targetQueryLine
}

proc tQueryTranslate { queryLine } {
   set pattern "^\[ \t\]*(.+)\[ \t\]+(holds-at)\[ \t\]+(.+)\[ \t\]*\$"
   regexp $pattern $queryLine match literal holds timepoint
   if {[regexp -- {-(.+)} $literal match literal] == 1} {
      return "neg(holds(${literal}, ${timepoint}))"
   } else {
      return "holds(${literal}, ${timepoint})"
   }
}



proc parse_domain { domainLine } {

   set literalPattern {-?[\_\=\<\>\\a-zA-Z0-9]+} ;# {-?[a-zA-Z][a-zA-Z0-9]*}
   set numberPattern {[0-9]+}
   set commentPattern {^%.*}
   set compoundLiteralPattern "${literalPattern}(\\(\[ \t\]*${literalPattern}\[ \t\]*(,\[ \t\]*${literalPattern})*\[ \t\]*\\))?"
   set compoundLiteralPattern "${literalPattern}(\\(\[ \t\]*${literalPattern}\[ \t\]*(,\[ \t\]*${compoundLiteralPattern})*\[ \t\]*\\))?"
   set literalsPattern "\{\[ \t\]*${compoundLiteralPattern}(\[ \t\]*,\[ \t\]*${compoundLiteralPattern})*\[ \t\]*\}"

   set constFluentPattern "^\[ \t\]*(${compoundLiteralPattern})\[ \t\]+(is_constant)\[ \t\]*\$"
   set tPattern "^\[ \t\]*(${compoundLiteralPattern})\[ \t\]+(holds-at)\[ \t\]+(${numberPattern})\[ \t\]*\$"
   set hPattern "^\[ \t\]*(${compoundLiteralPattern})\[ \t\]+(happens-at)\[ \t\]+(${numberPattern})\[ \t\]*\$"
   set rPattern "^\[ \t\]*(${compoundLiteralPattern})\[ \t\]+(whenever)\[ \t\]+(${literalsPattern})\[ \t\]*\$"
   set cPatternLong "^\[ \t\]*${compoundLiteralPattern}\[ \t\]+(initiates|terminates)\[ \t\]+${compoundLiteralPattern}\[ \t\]+(when)\[ \t\]+${literalsPattern}\[ \t\]*\$"
   set cPatternShort "^\[ \t\]*${compoundLiteralPattern}\[ \t\]+(initiates|terminates)\[ \t\]+${compoundLiteralPattern}\[ \t\]*\$"

   set targetDomainLine ""

   switch -regexp -- $domainLine \
	$commentPattern {} \
	$tPattern {set targetDomainLine [tPropositionTranslate $domainLine]} \
	$hPattern {set targetDomainLine [hPropositionTranslate $domainLine]} \
	$rPattern {set targetDomainLine [rPropositionTranslate $domainLine]} \
	$cPatternLong {set targetDomainLine [cPropositionLongTranslate $domainLine]} \
	$cPatternShort {set targetDomainLine [cPropositionShortTranslate $domainLine]} \
	$constFluentPattern {set targetDomainLine [constFluentTranslate $domainLine]}\
	"^\[ \t\]*\$" {} \
	default {set targetDomainLine "Syntax error: $domainLine"}
   
   return $targetDomainLine
}






proc parse_action { actionLine } {

   set literalPattern {-?[\_\=\<\>\\a-zA-Z0-9]+} ; # {-?[a-zA-Z][a-zA-Z0-9]*}
   set numberPattern {[0-9]+}

   set compoundLiteralPattern "${literalPattern}(\\(\[ \t\]*${literalPattern}\[ \t\]*(,\[ \t\]*${literalPattern})*\[ \t\]*\\))?"
   set compoundLiteralPattern "${literalPattern}(\\(\[ \t\]*${literalPattern}\[ \t\]*(,\[ \t\]*${compoundLiteralPattern})*\[ \t\]*\\))?"
   set literalsPattern "\{\[ \t\]*${compoundLiteralPattern}(\[ \t\]*,\[ \t\]*${compoundLiteralPattern})*\[ \t\]*\}"

	set aPattern "^\[ \t\]*(${compoundLiteralPattern})\[ \t\]+(is_abducible)\[ \t\]*\$"
        set ppropPattern "^\[ \t\]*(${compoundLiteralPattern})\[ \t\]+(needs)\[ \t\]+(${literalsPattern})\[ \t\]*\$"
	set prPattern "^\[ \t\]*${compoundLiteralPattern}\[ \t\]+(has_priority)\[ \t\]+${compoundLiteralPattern}\[ \t\]+(for)\[ \t\]+${literalPattern}\[ \t\]*\$"
 
	set targetActionLine ""

   switch -regexp -- $actionLine \
	$aPattern {set targetActionLine [aActionTranslate $actionLine]} \
        $ppropPattern {set targetActionLine [pActionTranslate $actionLine]} \
	$prPattern {set targetActionLine [priorityTranslate $actionLine]} \
	"^\[ \t\]*\$" {} \
	default {set targetActionLine "Syntax error: $actionLine"}

   return $targetActionLine

}




proc aActionTranslate { actionLine } {
   set pattern "^\[ \t\]*(.+)\[ \t\]+(is_abducible)\[ \t\]*\$"
   regexp $pattern $actionLine match action isabducible
   return "abducible($action)."
}


proc pActionTranslate { actionLine } {
   upvar compoundLiteralPattern compoundLiteralPatt
   set pattern "^\[ \t\]*(.+)\[ \t\]+(needs)\[ \t\]+\{(.+)\}\[ \t\]*\$"
   regexp $pattern $actionLine match action needs literals

   set tempStr ""
   set tempStr2 ""
   regsub -all "\[ \t\]*(${compoundLiteralPatt})\[ \t\]*" $literals "\n\t holds(&! T)" tempStr
   regsub -all {holds\([ ]*-([^!]+)! T\)} $tempStr {neg(holds(\1, T))} tempStr2
   regsub -all {holds\([ ]*([^!]+)! T\)} $tempStr2 {holds(\1, T)} tempStr

   set targetStr "pprop($action, T, \[$tempStr\])."

   set targetStr [join $targetStr ""]

   return $targetStr
}


proc priorityTranslate { actionLine } {
   upvar compoundLiteralPattern compoundLiteralPatt
   set pattern "^\[ \t\]*(.+)\[ \t\]+(has_priority)\[ \t\]+(.+)\[ \t\]+(for)\[ \t\]+(.+)\[ \t\]*\$"
   regexp $pattern $actionLine match action1 haspriority action2 for literal

   set targetStr "priority($literal, $action1, $action2)."

   return $targetStr
}




proc tPropositionTranslate { domainLine } {
   set pattern "^\[ \t\]*(.+)\[ \t\]+(holds-at)\[ \t\]+(.+)\[ \t\]*\$"
   regexp $pattern $domainLine match literal holds timepoint
   if {[regexp -- {-(.+)} $literal match literal] == 1} {
      return "tprop(neg(holds(${literal}, ${timepoint})))."
   } else {
      return "tprop(holds(${literal}, ${timepoint}))."
   }
}

proc hPropositionTranslate { domainLine } {
   set pattern "^\[ \t\]*(.+)\[ \t\]+(happens-at)\[ \t\]+(.+)\[ \t\]*\$"
   regexp $pattern $domainLine match action happens timepoint
   if {[regexp -- {-(.+)} $action match action] == 1} {
      return "neg(happens($action, $timepoint))."
   } else {
      return "happens($action, $timepoint)."
   }

}

proc rPropositionTranslate { domainLine } {
   upvar compoundLiteralPattern compoundLiteralPatt
   set pattern "^\[ \t\]*(.+)\[ \t\]+(whenever)\[ \t\]+\{(.+)\}\[ \t\]*\$"
   regexp $pattern $domainLine match literal whenever literals

   set tempStr ""
   set tempStr2 ""
   regsub -all "\[ \t\]*(${compoundLiteralPatt})\[ \t\]*" $literals " \n\t holds(&! T)" tempStr
   regsub -all {holds\([ ]*-([^!]+)! T\)} $tempStr {neg(holds(\1, T))} tempStr2
   regsub -all {holds\([ ]*([^!]+)! T\)} $tempStr2 {holds(\1, T)} tempStr
 
   set targetStr ""
   if {[regexp -- {-(.+)} $literal match literal] == 1} {
      lappend targetStr "ram(neg(holds($literal, T))) :- "
   } else {
      lappend targetStr "ram(holds($literal, T)) :- "
   }
   lappend targetStr $tempStr 
   lappend targetStr .

   set targetStr [join $targetStr ""] 

   return $targetStr
}


proc cPropositionLongTranslate { domainLine } {
   upvar compoundLiteralPattern compoundLiteralPatt
   set pattern "^\[ \t\]*(.+)\[ \t\]+(initiates|terminates)\[ \t\]+(.+)\[ \t\]+(when)\[ \t\]+\{(.+)\}\[ \t\]*\$"
   regexp $pattern $domainLine match action initOrTerm fluent when literals

   regsub {[ ]*(initiates)[ ]*} $initOrTerm "initiation" initOrTerm
   regsub {[ ]*(terminates)[ ]*} $initOrTerm "termination" initOrTerm

   set tempStr ""
   set tempStr2 ""
   set literals [join $literals ""]
   regsub -all "\[ \t\]*(${compoundLiteralPatt})\[ \t\]*" $literals " \n\t holds(&! T)" tempStr
   regsub -all {holds\([ ]*-([^!]+)! T\)} $tempStr {neg(holds(\1, T))} tempStr2
   regsub -all {holds\([ ]*([^!]+)! T\)} $tempStr2 {holds(\1, T)} tempStr
 
   set targetStr ""
   if {[regexp -- {-(.+)} $fluent match fluent] == 1} {
      lappend targetStr "${initOrTerm}(neg($fluent, T)) :- \n\t happens($action, T),"
   } else {
      lappend targetStr "${initOrTerm}($fluent, T) :- \n\t happens($action, T),"
   }
   lappend targetStr $tempStr 
   lappend targetStr ",\n\t true."		;# true is needed by E-RES

   set targetStr [join $targetStr ""] 

   return $targetStr
}

proc cPropositionShortTranslate { domainLine } {
   upvar compoundLiteralPattern compoundLiteralPatt
   set pattern "^\[ \t\]*(.+)\[ \t\]+(initiates|terminates)\[ \t\]+(.+)\[ \t\]*\$"
   regexp $pattern $domainLine match action initOrTerm fluent

   regsub {[ ]*(initiates)[ ]*} $initOrTerm "initiation" initOrTerm
   regsub {[ ]*(terminates)[ ]*} $initOrTerm "termination" initOrTerm

   if {[regexp -- {-(.+)} $fluent match fluent] == 1} {
      set targetStr "${initOrTerm}(neg($fluent, T)) :- \n\t happens($action, T), \n\t true."	;# true is needed by E-RES
   } else {
      set targetStr "${initOrTerm}($fluent, T) :- \n\t happens($action, T), \n\t true."	;# true is needed by E-RES
   }


   return $targetStr
}





proc constFluentTranslate { actionLine } {
   set pattern "^\[ \t\]*(.+)\[ \t\]+(is_constant)\[ \t\]*\$"
   regexp $pattern $actionLine match fluent isconstant
   return "constantfluent($fluent)."
}




