% =============================================================================================
% Computes the effects of all the actions.
% =============================================================================================

domain_pre_processing:-
	show('\n\n\n>> ====================\n>> Domain pre-processing...\n>> ====================\n\n\n'),
	find_all_rams(Rams), find_all_actions(Actions),
	show('\n>> Finding action effects:\n'),
	find_action_effects(Rams, Actions).


% =============================================================================================
% Computes the relevant part of the domain, according to the query.
% =============================================================================================

query_pre_processing([], Relevant):-
	!,	% No Query. Assimilate Observations.
	show('\n\n\n>> ====================\n>> Assimilating Observations...\n>> ====================\n\n\n'),
	find_all_tprops(Tprops),
	show('\n>> Observations:\n'), showList(Tprops),
	set_all_times(Tprops, T, TpropsT),
	query_pre_processing_aux(TpropsT, Relevant).

query_pre_processing(Query, Relevant):-
	show('\n\n\n>> ====================\n>> Query pre-processing...\n>> ====================\n\n\n'),
	show('\n>> Input Query list:\n'), showList(Query),
	set_all_times(Query, T, QueryT),
	query_pre_processing_aux(QueryT, Relevant).


query_pre_processing_aux(Seed, relevant(RelevantTprops, RelevantRams, RelevantActions, RelevantLiterals, RelevantTimes)):-
	show('\n>> Finding relevant part of domain:\n'),
	find_relevant_literals(Seed, RelevantLiterals, RelevantRams, RelevantActions),
	show('\n>> Relevant literals: '),
	length(RelevantLiterals, L), show([L, '\n']),
	showList(RelevantLiterals),
	show('\n>> Relevant ramifications: '),
	length(RelevantRams, R), show([R, '\n']),
	showList(RelevantRams),
	show('\n>> Relevant actions: '),
	length(RelevantActions, A), show([A, '\n']),
	showList(RelevantActions),
	show('\n>> Relevant time points: '),
	find_relevant_time_points(RelevantActions, RelevantTimes),
	show([RelevantTimes, '\n']),
	show('\n>> Finding relevant t-propositions:\n'),
	find_relevant_tprops(RelevantLiterals, RelevantTprops1),
	showList(RelevantTprops1),
	show('\n>> Promoting relevant t-propositions:\n'),
	promote_literals(RelevantTprops1, RelevantTimes, RelevantTprops),
	showList(RelevantTprops).


% =============================================================================================
% End of file.
% =============================================================================================
