% =============================================================================================
% Skeptically projects Delta at the given time point.
% =============================================================================================

skeptically_project_Delta_at_Time(Delta, T, ProjectedLiterals):- 
	convert_Delta_to_Literals_at_T(Delta, T, DeltaLiterals),
	find_skeptically_projected_Literals(Delta, DeltaLiterals, ProjectedLiterals).


% =============================================================================================
% Finds which Literals hold skeptically (positively or negatively), given Delta.
% =============================================================================================

find_skeptically_projected_Literals(Delta, Literals, ProjectedLiterals):-
	find_skeptically_projected_Literals_aux(Delta, Literals, ProjectedLiterals1),
	removeduplicates(ProjectedLiterals1, ProjectedLiterals).


find_skeptically_projected_Literals_aux(_Delta, [], []).

find_skeptically_projected_Literals_aux(_, [_L|_], _):-
	%	show(_L),
	fail.

find_skeptically_projected_Literals_aux(Delta, [L|Literals], [L|Projected]):-
	once(abd(Delta, [L], _)), contrary(L, NL), \+ abd(Delta, [NL], _), !,
	%	show(['-pos', '\n']),		% Project positive skeptically.
	find_skeptically_projected_Literals_aux(Delta, Literals, Projected).

find_skeptically_projected_Literals_aux(Delta, [L|Literals], [NL|Projected]):-
	contrary(L, NL), once(abd(Delta, [NL], _)), \+ abd(Delta, [L], _), !,
	%	show(['-neg', '\n']),		% Project negative skeptically.
	find_skeptically_projected_Literals_aux(Delta, Literals, Projected).

find_skeptically_projected_Literals_aux(Delta, [L|Literals], Projected):-
	once(abd(Delta, [L], _D1)), contrary(L, NL), abd(Delta, [NL], _D2), !,
	%	show(['-none', '\n']),		% Cannot be projected skeptically.
	find_skeptically_projected_Literals_aux(Delta, Literals, Projected).

find_skeptically_projected_Literals_aux(_Delta, [L|_Literals], _Projected):-
	show(['>> No model exists for this Delta, due to literal: ', L, '.\n']),
	fail.


% =============================================================================================
% End of file.
% =============================================================================================
