% =============================================================================================
% Useful predicates.
% =============================================================================================

get_fluent(neg(holds(F,_)), F).
get_fluent(holds(F,_), F).


% Change definition to also include all abducible actions.
action_happens(happens(F,_T)):-
	clause(happens(F,_),true).	% Protect time variables from unification.


make_all_positive([], []).
make_all_positive([X|Rest], [holds(F,T)|PosRest]):-
	(neg(holds(F,T))=X ; holds(F,T)=X),
	make_all_positive(Rest, PosRest).


common_element(List1, List2):-
	mymember(X, List1),
	mymember(X, List2), !.
	

promote_literals([], _RelevantTimes, []).
promote_literals([One|Rest], RelevantTimes, [PromotedOne|PromotedRest]):-
	promote_literals_one(One, RelevantTimes, PromotedOne),
	promote_literals(Rest, RelevantTimes, PromotedRest).

promote_literals_one(One, RelevantTimes, One):-
	gettime(One, T), mymember(T, RelevantTimes), !.
promote_literals_one(One, RelevantTimes, PromotedOne):-
	gettime(One, T), T1 is T-1, settime(T1, One, NOne),
promote_literals_one(NOne, RelevantTimes, PromotedOne).

 
find_literals_at_time([], _T, []).
find_literals_at_time([One|Rest], T, [One|RestAtT]):-
	settime(T, One, One), !,
	find_literals_at_time(Rest, T, RestAtT).
find_literals_at_time([_|Rest], T, RestAtT):-
	find_literals_at_time(Rest, T, RestAtT).


list_append([], []).
list_append([X|Rest], Result):-
	list_append(Rest, Result1),
	myappend(X, Result1, Result).


gettime(holds(_,T), T).
gettime(neg(holds(_,T)), T).

set_all_times([], _NT, []).
set_all_times([L|Literals], NT, [NL|NLiterals]):-
	settime(NT, L, NL),
	set_all_times(Literals, NT, NLiterals).
	
contrary_all([], []).
contrary_all([X|Rest], [NX|NRest]):-
	contrary(X, NX),
	contrary_all(Rest, NRest).


convert_tprops_to_rams([], []).
convert_tprops_to_rams([One|Tprops], [ramification(One,[])|Rams]):-
	convert_tprops_to_rams(Tprops, Rams).


add_Literals_to_Delta(Delta, [], Delta).
add_Literals_to_Delta(Delta, [L|Literals], [R|Delta1]):-
	convert_rule_to_literal(R, L),
	add_Literals_to_Delta(Delta, Literals, Delta1).


convert_Delta_to_Literals_at_T(Delta, T, LiteralsT):-
	convert_Delta_to_Literals_aux(Delta, Literals),
	set_all_times(Literals, T, Literals1),
	removeduplicates(Literals1, LiteralsT).

convert_Delta_to_Literals(Delta, Literals):-
	convert_Delta_to_Literals_aux(Delta, Literals1),
	removeduplicates(Literals1, Literals).

convert_Delta_to_Literals_aux([], []).
convert_Delta_to_Literals_aux([D|Delta], Literals):-
	rule_contains_constant_fluent(D), !, 
	convert_Delta_to_Literals_aux(Delta, Literals).
convert_Delta_to_Literals_aux([D|Delta], [L|Literals]):-
	convert_rule_to_literal(D, L),
	convert_Delta_to_Literals_aux(Delta, Literals).

convert_rule_to_literal(rule(ass, neg(F), T), neg(holds(F, T))):- !.
convert_rule_to_literal(rule(ass, F, T), holds(F, T)):- !.
convert_rule_to_literal(rule(gen, neg(F), T, _X), neg(holds(F, T))):- !.
convert_rule_to_literal(rule(gen, F, T, _X), holds(F, T)):- !.

rule_contains_constant_fluent(rule(ass, neg(F), _T)):- !, constantfluent(F).
rule_contains_constant_fluent(rule(ass, F, _T)):- !, constantfluent(F).
rule_contains_constant_fluent(rule(gen, neg(F), _T, _)):- !, constantfluent(F).
rule_contains_constant_fluent(rule(gen, F, _T, _)):- !, constantfluent(F).

invert_rule(rule(ass, neg(F), T), rule(ass, F, T)):- !.
invert_rule(rule(ass, F, T), rule(ass, neg(F), T)):- !.
invert_rule(rule(gen, neg(F), T, X), rule(gen, F, T, X)):- !.
invert_rule(rule(gen, F, T, X), rule(gen, neg(F), T, X)):- !.


list_consistent([], _Literals).
list_consistent([L|Rest], Literals):-
	is_consistent(L, Literals), !,
	list_consistent(Rest, [L|Literals]).
list_consistent([L|_], _):-
	show(['>> Offending fluent: ', L, '\n']),
	fail.

is_consistent(L, Literals):-
	L \= neg(holds(true,_)),
	contrary(L, NL),
	\+ mymember(NL, Literals).

check_if_literals_can_be_added_to_Delta(Delta, Literals):-
	once(abd(Delta, Literals, _)).

check_if_literals_can_be_added_to_Delta_2(Delta, Literals):-
	convert_Delta_to_Literals(Delta, DeltaLiterals),
	myappend(Literals, DeltaLiterals, AllLiterals),	
	once(abd([], AllLiterals, _)).


find_support_for_adding_literals_to_Delta(Delta, Literals, SupportLiterals):-
	abd(Delta, Literals, Delta1),
	subtract(Delta1, Delta, NewDelta),
	convert_Delta_to_Literals(NewDelta, SupportLiterals).


say([]):- !, flush(stdout).
say([X|Rest]):- !, write(X), say(Rest).
say(X):- say([X]).


% show_messages/1 is defined in e-res.tcl

show(X):- (show_messages(all);show_messages(nolists)), !, say(X).
show(_).

showList(X):- show_messages(all), !, writelist(X).
showList(X):- show_messages(nolists), !, length(X,N), say(['{ list size: ', N, ' }\n']).
showList(_).

showDelta(X):- show_messages(all), !, writeDelta(X).
showDelta(X):- show_messages(nolists), !, length(X,N), say(['{ list size: ', N, ' }\n']).
showDelta(_).

writeDelta([]).
writeDelta([X|List]):-
	rule_contains_constant_fluent(X), !, 
	writeDelta(List).
writeDelta([X|List]):-
	write(X), nl,
	writeDelta(List).


% =============================================================================================
% End of file.
% =============================================================================================
