% =============================================================================================
% Determines the computation to take place in order to validate the domain constraints.
% =============================================================================================

checkallrams(Delta, relevant(_Tprops, _Rams, _Actions, _, []), Delta).
checkallrams(Delta, relevant(Tprops, Rams, Actions, _, [T|RestTimes]), Delta1):-
	show(['\n\n>> Time point: ', T, '\n', '>> ===========\n']),
	show('\n>> Input Delta for time point:\n'), showDelta(Delta),
	find_full_set_of_skeptical_literals(Delta, Tprops, Rams, Actions, T, Delta2),
	checkallrams(Delta2, relevant(Tprops, Rams, Actions, _, RestTimes), Delta1).


% =============================================================================================
% Finds the full set of Literlas that hold skeptically at the given time point.
% =============================================================================================

find_full_set_of_skeptical_literals(Delta, Tprops, Rams, Actions, T, Delta1):-
	find_literals_at_time(Tprops, T, TpropsAtT),
	convert_tprops_to_rams(TpropsAtT, TpropsRams),
	myappend(TpropsRams, Rams, AllRams),
	show('\n>> Skeptically projecting Delta:\n'),
	convert_Delta_to_Literals(Delta, DeltaLiterals),
	show('\n>> Last values of literals:\n'),
	myappend(Tprops, DeltaLiterals, DeltaLiterals1), % Take tprops into account.
	find_last_values_of_literals(DeltaLiterals1, LiteralsLastValues),
	showList(LiteralsLastValues),
	show('\n>> Affected literals:\n'),
	find_affected_literals(LiteralsLastValues, T, AffectedLiteralsT, UnaffectedLiteralsT),
	showList(AffectedLiteralsT),
	show('\n>> Unaffected literals:\n'),
	showList(UnaffectedLiteralsT),
	show('\n>> Computing Skeptical Affected Literals:\n'),
%	contrary_all(AffectedLiteralsT, NAffectedLiteralsT), % Efficiency improvement?
	find_skeptically_projected_Literals(Delta, AffectedLiteralsT, SkepticalAffectedLiterals),
	showList(SkepticalAffectedLiterals),
	show('\n>> Skeptical conclusions:\n'),
%	add_Literals_to_Delta(Delta, SkepticalAffectedLiterals, Delta2),
	myappend(SkepticalAffectedLiterals, UnaffectedLiteralsT, SkepticalLiterals),
	find_full_set_of_skeptical_literals_aux(Delta, AllRams, Actions, SkepticalLiterals, Delta1).

find_full_set_of_skeptical_literals_aux(Delta, Rams, Actions, SkepticalLiterals, Delta1):-
	showList(SkepticalLiterals),
	show('\n>> Computing Relevant Literals:\n'),
	find_relevant_at_depth_one(SkepticalLiterals, Rams, Actions, PosRelLiterals),
	showList(PosRelLiterals),
	show('\n>> Computing Forced Literals:\n'),
	find_forced_literals(Rams, SkepticalLiterals, AllForcedLiterals),
	contrary_all(PosRelLiterals, NegRelLiterals),
	intersection(AllForcedLiterals, PosRelLiterals, PosForcedLiterals1),
	subtract(PosForcedLiterals1, SkepticalLiterals, PosForcedLiterals),
	intersection(AllForcedLiterals, NegRelLiterals, NegForcedLiterals1),
	subtract(NegForcedLiterals1, SkepticalLiterals, NegForcedLiterals),
	myappend(PosForcedLiterals, NegForcedLiterals, ForcedLiterals),
	showList(ForcedLiterals),
	show('\n>> Adding Literals to Delta...\n'),
	check_if_literals_can_be_added_to_Delta(Delta, ForcedLiterals), !,
	augment_Delta(Delta, PosForcedLiterals, NegForcedLiterals, Rams, Actions, SkepticalLiterals, PosRelLiterals, Delta1).
find_full_set_of_skeptical_literals_aux(_, _, _, _, _):-
	show('\n>> Inconsistency detected while computing skeptical/forced literals. Backtracking...\n\n\n'),
	fail.

augment_Delta(Delta, PosForcedLiterals, NegForcedLiterals, Rams, Actions, PreviousSkepticalLiterals, PosRelLiterals, Delta1):-
	add_Literals_to_Delta([], PosForcedLiterals, AddDelta1),
	add_Literals_to_Delta(AddDelta1, NegForcedLiterals, AddDelta),
	subtract(AddDelta, Delta, NewDelta),
	show('\n>> New Literals added to Delta:\n'), showList(NewDelta),
	myappend(NewDelta, Delta, Delta2),
	show('\n>> New Delta List:\n'), showList(Delta2),
	show('\n>> Computing Skeptical Literals:\n'),
	contrary_all(NegForcedLiterals, NNegForcedLiterals),
	subtract(PosRelLiterals, PosForcedLiterals, RestRelLiterals1),
	subtract(RestRelLiterals1, NNegForcedLiterals, RestRelLiterals2),
	subtract(RestRelLiterals2, PreviousSkepticalLiterals, RestRelLiterals),
	find_skeptically_projected_Literals(Delta2, RestRelLiterals, RestSkepticalLiterals),
	list_append([PreviousSkepticalLiterals, PosForcedLiterals, NegForcedLiterals, RestSkepticalLiterals], SkepticalLiterals1),
	removeduplicates(SkepticalLiterals1, SkepticalLiterals),
	subtract(SkepticalLiterals, PreviousSkepticalLiterals, [_|_]), !,
	find_full_set_of_skeptical_literals_aux(Delta2, Rams, Actions, SkepticalLiterals, Delta1).
augment_Delta(Delta, PosForcedLiterals, NegForcedLiterals, _, _, _, _, Delta1):-
	add_Literals_to_Delta([], PosForcedLiterals, AddDelta1),
	add_Literals_to_Delta(AddDelta1, NegForcedLiterals, AddDelta),
	subtract(AddDelta, Delta, NewDelta),
	myappend(NewDelta, Delta, Delta1),
	show('\n>> All skeptical/forced literals have been computed.\n\n').


% =================

find_relevant_at_depth_one(SeedLiterals, Rams, Actions, RelLiterals):-
	find_relevant_from_rams(SeedLiterals, Rams, RamsRelevant),
	find_relevant_from_actions(SeedLiterals, Actions, ActionsRelevant),
	list_append([SeedLiterals, ActionsRelevant, RamsRelevant], RelLiterals1),
	make_all_positive(RelLiterals1, RelLiterals2),
	removeduplicates(RelLiterals2, RelLiterals).
	
find_relevant_from_rams(Literals, Rams, RamsRelevant):-
	findall(GroundRam, (mymember(ramification(H,B),Rams), contrary_all([H|B],NHB), myappend([H|B],NHB,RamLit), common_element(RamLit,Literals), ground([H|B],GroundRam)), GroundedOneRams),
	list_append(GroundedOneRams, RamsRelevant).

find_relevant_from_actions(Literals, Actions, ActionsRelevant):-
	findall(GroundAct, (mymember(action(A,H,B),Actions), action_happens(A), contrary_all([H|B],NHB), myappend([H|B],NHB,ActLit), common_element(ActLit,Literals), ground([H|B],GroundAct)), GroundedOneActions),
	list_append(GroundedOneActions, ActionsRelevant).


% =================

find_last_values_of_literals(DeltaLiterals, LiteralsLastValues):-
	myselect(X, DeltaLiterals, DeltaLiterals1),
      (holds(F,TX)=X;neg(holds(F,TX))=X),
	mymember(Y, DeltaLiterals1),
      (holds(F,TY)=Y;neg(holds(F,TY))=Y),
	TX<TY, !,
	find_last_values_of_literals(DeltaLiterals1, LiteralsLastValues).
find_last_values_of_literals(DeltaLiterals, DeltaLiterals1):-
	removeduplicates(DeltaLiterals, DeltaLiterals1).


find_affected_literals(LiteralsPrevValues, T, AffectedLiteralsT, UnaffectedLiteralsT):-
	findall(L, (mymember(L,LiteralsPrevValues), gettime(L,T1), settime(_,L,L1), contrary(L1,NL),
			(
			 T=<T1 %(T<T1, once((get_action_effect_info(L1,T2,_,_), T=<T2, T2<T1)))
			 ;
 		  	 (T1<T, once((get_action_effect_info(NL,T2,_,_), T1=<T2, T2<T)))
			)), AffectedLiterals),
	set_all_times(LiteralsPrevValues, T, LiteralsPrevValuesT),
	set_all_times(AffectedLiterals, T, AffectedLiteralsT),
	subtract(LiteralsPrevValuesT, AffectedLiteralsT, UnaffectedLiteralsT).

% =============================================================================================
% End of file.
% =============================================================================================
